This is the mail archive of the binutils@sourceware.org mailing list for the binutils project.


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
Other format: [Raw text]

Re: PATCH: Don't match PE/EFI target with EFI/PE file


On Fri, Jun 08, 2007 at 10:53:50AM -0700, H. J. Lu wrote:
> On Wed, Jun 06, 2007 at 04:34:12PM -0700, H. J. Lu wrote:
> > When both PE and EFI are enabled for i386/x86-64, objdump will
> > match both PE and EFI targets with PE/EFI input files. This
> > patch checks IMAGE_SUBSYSTEM_EFI_APPLICATION to avoid matching
> > PE/EFI target with EFI/PE file.
> > 
> 
> I updated the patch to fix a logic error and support x86-64. Tested
> on Win32 and Win64 executables. OK to install?
> 

Change EFI target names may break existing scripts. This updated
patch leaves them alone.


H.J.
----
2007-06-09  H.J. Lu  <hongjiu.lu@intel.com>

	* libpei.h (_bfd_XXi_final_link_postscript): Remove
	duplication.
	(bfd_target_pei_p): New.
	(bfd_target_pei_arch): New
	(bfd_target_efi_p): Likewise.
	(bfd_target_efi_arch): New
	(bfd_pe_executable_p): Use bfd_target_pei_p and
	bfd_target_efi_p.

	* peicode.h (arch_type): New enum.
	(pe_arch): New function.
	(pe_bfd_object_p): Don't match PE/EFI target with EFI/PE file
	if there is an EFI/PE target.

--- bfd/peicode.h.efi	2006-09-20 04:35:07.000000000 -0700
+++ bfd/peicode.h	2007-06-09 07:12:12.000000000 -0700
@@ -1234,6 +1234,25 @@ pe_ILF_object_p (bfd * abfd)
   return abfd->xvec;
 }
 
+enum arch_type
+{
+  unknown,
+  i386,
+  x86_64
+};
+
+static enum arch_type
+pe_arch (const char *arch)
+{
+  if (strcmp (arch, "i386") == 0 || strcmp (arch, "ia32") == 0)
+    return i386;
+
+  if (strcmp (arch, "x86_64") == 0 || strcmp (arch, "x86-64") == 0)
+    return x86_64;
+
+  return unknown;
+}
+
 static const bfd_target *
 pe_bfd_object_p (bfd * abfd)
 {
@@ -1241,6 +1260,7 @@ pe_bfd_object_p (bfd * abfd)
   struct external_PEI_DOS_hdr dos_hdr;
   struct external_PEI_IMAGE_hdr image_hdr;
   file_ptr offset;
+  const bfd_target *target;
 
   /* Detect if this a Microsoft Import Library Format element.  */
   if (bfd_seek (abfd, (file_ptr) 0, SEEK_SET) != 0
@@ -1305,7 +1325,60 @@ pe_bfd_object_p (bfd * abfd)
       return NULL;
     }
 
-  return coff_object_p (abfd);
+  target = coff_object_p (abfd);
+  if (target)
+    {
+      pe_data_type *pe = pe_data (abfd);
+      struct internal_extra_pe_aouthdr *i = &pe->pe_opthdr;
+      bfd_boolean efi = i->Subsystem == IMAGE_SUBSYSTEM_EFI_APPLICATION;
+      enum arch_type arch;
+      const bfd_target * const *target_ptr;
+
+      /* Get the machine.  */
+      if (bfd_target_efi_p (abfd->xvec))
+	arch = pe_arch (bfd_target_efi_arch (abfd->xvec));
+      else
+	arch = pe_arch (bfd_target_pei_arch (abfd->xvec));
+
+      for (target_ptr = bfd_target_vector; *target_ptr != NULL;
+	   target_ptr++)
+	{
+	  if (*target_ptr == target
+	      || (*target_ptr)->flavour != bfd_target_coff_flavour)
+	    continue;
+
+	  if (bfd_target_efi_p (*target_ptr))
+	    {
+	      /* Skip incompatible arch.  */
+	      if (pe_arch (bfd_target_efi_arch (*target_ptr)) != arch)
+		continue;
+
+		if (efi)
+		  {
+		    /* TARGET_PTR is an EFI backend.  Don't match
+		       TARGET with a EFI file.  */
+		    bfd_set_error (bfd_error_wrong_format);
+		    return NULL;
+		  }
+	    }
+	  else if (bfd_target_pei_p (*target_ptr))
+	    {
+	      /* Skip incompatible arch.  */
+	      if (pe_arch (bfd_target_pei_arch (*target_ptr)) != arch)
+		continue;
+
+		if (!efi)
+		  {
+		    /* TARGET_PTR is a PE backend.  Don't match
+		       TARGET with a PE file.  */
+		    bfd_set_error (bfd_error_wrong_format);
+		    return NULL;
+		  }
+	    }
+	}
+    }
+
+  return target;
 }
 
 #define coff_object_p pe_bfd_object_p


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
--- bfd/libpei.h.efi	2006-09-20 04:35:07.000000000 -0700
+++ bfd/libpei.h	2007-06-09 07:11:25.000000000 -0700
@@ -224,7 +224,6 @@
 #define _bfd_XX_only_swap_filehdr_out			_bfd_pex64_only_swap_filehdr_out
 #define _bfd_XX_print_private_bfd_data_common		_bfd_pex64_print_private_bfd_data_common
 #define _bfd_XXi_final_link_postscript			_bfd_pex64i_final_link_postscript
-#define _bfd_XXi_final_link_postscript			_bfd_pex64i_final_link_postscript
 #define _bfd_XXi_only_swap_filehdr_out			_bfd_pex64i_only_swap_filehdr_out
 #define _bfd_XXi_swap_aouthdr_in			_bfd_pex64i_swap_aouthdr_in
 #define _bfd_XXi_swap_aouthdr_out			_bfd_pex64i_swap_aouthdr_out
@@ -256,7 +255,6 @@
 #define _bfd_XX_only_swap_filehdr_out			_bfd_pep_only_swap_filehdr_out
 #define _bfd_XX_print_private_bfd_data_common		_bfd_pep_print_private_bfd_data_common
 #define _bfd_XXi_final_link_postscript			_bfd_pepi_final_link_postscript
-#define _bfd_XXi_final_link_postscript			_bfd_pepi_final_link_postscript
 #define _bfd_XXi_only_swap_filehdr_out			_bfd_pepi_only_swap_filehdr_out
 #define _bfd_XXi_swap_aouthdr_in			_bfd_pepi_swap_aouthdr_in
 #define _bfd_XXi_swap_aouthdr_out			_bfd_pepi_swap_aouthdr_out
@@ -288,7 +286,6 @@
 #define _bfd_XX_only_swap_filehdr_out			_bfd_pe_only_swap_filehdr_out
 #define _bfd_XX_print_private_bfd_data_common		_bfd_pe_print_private_bfd_data_common
 #define _bfd_XXi_final_link_postscript			_bfd_pei_final_link_postscript
-#define _bfd_XXi_final_link_postscript			_bfd_pei_final_link_postscript
 #define _bfd_XXi_only_swap_filehdr_out			_bfd_pei_only_swap_filehdr_out
 #define _bfd_XXi_swap_aouthdr_in			_bfd_pei_swap_aouthdr_in
 #define _bfd_XXi_swap_aouthdr_out			_bfd_pei_swap_aouthdr_out
@@ -302,10 +299,27 @@
 
 #endif /* !COFF_WITH_pep */
 
-/* Macro: Returns true if the bfd is a PE executable as opposed to a PE object file.  */
+/* Returns true if the target is a PE executable target.  */
+#define bfd_target_pei_p(xvec)				\
+  (CONST_STRNEQ ((xvec)->name, "pei-"))
+
+/* Return the arch string of a PE executable target.  */
+#define bfd_target_pei_arch(xvec)				\
+  ((xvec)->name + sizeof ("pei-") - 1)
+
+/* Returns true if the target is an EFI target.  */
+#define bfd_target_efi_p(xvec)				\
+   (CONST_STRNEQ ((xvec)->name, "efi-app-"))
+
+/* Return the arch string of an EFI target.  */
+#define bfd_target_efi_arch(xvec)				\
+  ((xvec)->name + sizeof ("efi-app-") - 1)
+
+/* Macro: Returns true if the bfd is a PE executable as opposed to a
+   PE object file.  */
 #define bfd_pe_executable_p(abfd)			\
-  (CONST_STRNEQ ((abfd)->xvec->name, "pei-")		\
-   || CONST_STRNEQ ((abfd)->xvec->name, "efi-app-"))
+  (bfd_target_pei_p ((abfd)->xvec)			\
+   || bfd_target_efi_p ((abfd)->xvec))
 
 /* These functions are architecture dependent, and are in peicode.h:
    coff_swap_reloc_in